@@ -89,20 +89,12 @@ class DistutilsMetaFinder:
         return method()
 
     def spec_for_distutils(self):
+        if self.is_cpython():
+            return
+
         import importlib
         import importlib.abc
         import importlib.util
-        import warnings
-
-        # warnings.filterwarnings() imports the re module
-        warnings._add_filter(
-            'ignore',
-            _TrivialRe("distutils", "deprecated"),
-            DeprecationWarning,
-            None,
-            0,
-            append=True
-        )
 
         try:
             mod = importlib.import_module('setuptools._distutils')
@@ -125,7 +117,17 @@ class DistutilsMetaFinder:
             def exec_module(self, module):
                 pass
 
-        return importlib.util.spec_from_loader('distutils', DistutilsLoader())
+        return importlib.util.spec_from_loader(
+            'distutils', DistutilsLoader(), origin=mod.__file__
+        )
+
+    @staticmethod
+    def is_cpython():
+        """
+        Suppress supplying distutils for CPython (build and tests).
+        Ref #2965 and #3007.
+        """
+        return os.path.isfile('pybuilddir.txt')
 
     def spec_for_pip(self):
         """
@@ -134,8 +136,6 @@ class DistutilsMetaFinder:
         """
         if self.pip_imported_during_build():
             return
-        if self.is_get_pip():
-            return
         clear_distutils()
         self.spec_for_distutils = lambda: None
 
@@ -150,17 +150,6 @@ class DistutilsMetaFinder:
             for frame, line in traceback.walk_stack(None)
         )
 
-    @classmethod
-    def is_get_pip(cls):
-        """
-        Detect if get-pip is being invoked. Ref #2993.
-        """
-        try:
-            import __main__
-            return os.path.basename(__main__.__file__) == 'get-pip.py'
-        except AttributeError:
-            pass
-
     @staticmethod
     def frame_file_is_setup(frame):
         """
